# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation; either version 2 of the License, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
""" Copyright (c) 2000-2003 LOGILAB S.A. (Paris, FRANCE).
 http://www.logilab.fr/ -- mailto:contact@logilab.fr

HTML reporter
"""

__revision__ = "$Id: html.py,v 1.5 2003/12/23 13:45:37 syt Exp $"

import sys
from logilab.pylint.interfaces import IReporter
from logilab.pylint.reporters import BaseReporter
from logilab.pylint.layouts import SectionLayout, TableLayout

from cgi import escape

class HTMLReporter(BaseReporter):
    """report messages and layouts in HTML
    """

    __implements____ = IReporter

    def __init__(self, output=sys.stdout):
        BaseReporter.__init__(self, output)
        self.msgs = []

    def add_message(self, msg_id, location, msg):
        """manage message of different type and in the context of path
        """
        module, obj, line = location
        if type == 'info':
            # print informal messages on stderr
            print >> sys.stderr, msg
        else:
            
            if self.include_ids:
                sigle = msg_id
            else:
                sigle = msg_id[0]
            self.msgs.append((sigle, module, obj, str(line), escape(msg)))

    def display_results(self, layout):
        """launch layouts display
        
        overriden from BaseReporter to display messages
        """
        if self.msgs:
            # add stored messages to the layout
            self.msgs.insert(0, ('type', 'module', 'object', 'line', 'message'))
            sect = SectionLayout('Messages')
            layout.append(sect)
            sect.append(TableLayout(self.msgs, row_headers=1))
            self.msgs = []
        BaseReporter.display_results(self, layout)
                    
    def visit_sectionlayout(self, layout, close=0):
        """display a section as html, using div + h[section level]
        """
        BaseReporter.visit_sectionlayout(self, layout, close)
        if close:
            self.writeln('</div>')
        else:
            self.writeln('<div>')
            self.writeln('<h%s>%s</h%s>' % (self.section,
                                           layout.title.capitalize(),
                                           self.section))
            if layout.description:
                self.writeln('<p>')
                self.writeln(layout.description)
                self.writeln('</p>')

    def visit_tablelayout(self, layout):
        """display a table as html
        """
        self.writeln('<table>')
        for i in range(len(layout.data)):
            line = layout.data[i]
            self.writeln('<tr>')
            for j in range(len(line)):
                cell = line[j]
                if (layout.col_headers and i == 0) or \
                   (layout.col_headers and j == 0):
                    self.writeln('<th>%s</th>' % cell)
                else:
                    self.writeln('<td>%s</td>' % cell)
            self.writeln('</tr>')
        self.writeln('</table>')
        

    def visit_verbatimlayout(self, layout):
        """display a verbatim layout as html (using <pre>)
        """
        self.writeln('<pre>')
        self.writeln(layout.data)
        self.writeln('</pre>')

