# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation; either version 2 of the License, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
""" Copyright (c) 2000-2003 LOGILAB S.A. (Paris, FRANCE).
 http://www.logilab.fr/ -- mailto:contact@logilab.fr

Plain text reporter
"""

__revision__ = "$Id: text.py,v 1.7 2003/12/23 17:11:49 syt Exp $"

import sys
import os
from os.path import exists, join

from logilab.pylint.interfaces import IReporter
from logilab.pylint.reporters import BaseReporter

TITLE_UNDERLINES = ['', '=', '-', '.']


def modname_to_path(modname, prefix=os.getcwd() + os.sep):
    """transform a module name into a path
    """
    m = __import__(modname)
    return m.__file__.replace(prefix, '').replace('.pyc', '.py').replace('.pyo', '.py')


class TextReporter(BaseReporter):
    """reports messages and layouts in plain text
    """
    
    __implements____ = IReporter
    
    def __init__(self, output=sys.stdout):
        BaseReporter.__init__(self, output)
        self._modules = {}

    def add_message(self, msg_id, location, msg):
        """manage message of different type and in the context of path
        """
        module, obj, line = location
        if not self._modules.has_key(module):
            self.writeln('************* Module %s' % module)
            self._modules[module] = 1
        if obj:
            obj = ':%s' % obj
        if self.include_ids:
            sigle = msg_id
        else:
            sigle = msg_id[0]
        self.writeln('%s:%3s%s: %s' % (sigle, line, obj, msg))

    def display_results(self, layout):
        """launch layouts display
        
        overriden from BaseReporter to add a blank line...
        """
        self.writeln()
        BaseReporter.display_results(self, layout)
    
    def visit_sectionlayout(self, layout, close=0):
        """display a section as text
        """
        BaseReporter.visit_sectionlayout(self, layout, close)
        if not close:
            title = layout.title
            self.writeln(title)
            self.writeln(TITLE_UNDERLINES[self.section] * len(title))
            if layout.description:
                self.writeln()
                self.writeln(layout.description)
            if layout.children:
                self.writeln()
        else:
            self.writeln()
            self.writeln()

    def visit_tablelayout(self, layout):
        """display a table as text
        """
        cols_size = [size+1 for size in layout.get_columns_size()]
        format_strings = ' '.join(['%%-%ss'] * len(cols_size))
        format_strings = format_strings % tuple(cols_size)
        format_strings = format_strings.split(' ')
        for i in range(len(layout.data)):
            line = layout.data[i]
            for j in range(len(line)):
                cell = line[j]
                if (layout.row_headers and i == 0) or \
                   (layout.col_headers and j == 0):
                    self.write(format_strings[j] % cell.upper())
                else:
                    self.write(format_strings[j] % cell)
            self.writeln()

    def visit_verbatimlayout(self, layout):
        """display a verbatim layout as text (so difficult ;)
        """
        self.writeln(layout.data)


class TextReporter2(TextReporter):
    """a reporter very similar to TextReporter, but display messages in a form
    recognized by most text editors :
    
    <filename>:<linenum>:<msg>
    """

    def add_message(self, msg_id, location, msg):
        """manage message of different type and in the context of path
        """
        module, obj, line = location
        if obj:
            obj = ', %s' % obj
        if self.include_ids:
            sigle = msg_id
        else:
            sigle = msg_id[0]
        try:
            modpath = self._modules[module]
        except:
            modpath = self._modules[module] = modname_to_path(module)
        self.writeln('%s:%s: [%s%s] %s' % (modpath, line, sigle, obj, msg))

    
